# Introduction

The Truffle implementation of the R FFI is based on the Truffle implementation of LLVM intermediate code, named [Sulong](https://github.com/graalvm/sulong).


# Building
Special setup is required to build FastR to use the Truffle R FFI implementation.

## Pre-Requisites

The DragonEgg plugin requires that `gcc 4.6` and `gfortran 4.6` be available.

On Mac OS, these can be installed with MacPorts (recommended) or Brew. Having installed these, set the following environment variables:

    export SULONG_GPP=/opt/local/bin/g++-mp-4.6
    export SULONG_GFORTRAN=/opt/local/bin/gfortran-mp-4.6
    export SULONG_GCC=/opt/local/bin/gcc-mp-4.6

The above definitions assume a MacPorts installation.

On Linux, the installation is system  dependent, but once installed, set the same environment variables.

## Building Sulong
The `sulong` repository must be cloned to a sibling directory of `fastr` and built:

    cd $FASTR_HOME
    git clone https://github.com/graalvm/sulong.git
    cd sulong
    mx su-pulldragonegg

The `mx su-pulldragonegg` step is required to be able to compile Fortran code to LLVM, which is required by FastR.. As well as downloading DragonEgg this will also download and clang 3.2, which is needed to build DragonEgg. On Linux, it is necessary to use clang 3.2 in preference to any installed clang, e.g., 3.8, as some of the LLVM code generated by DragonEgg is syntax-incompatible with 3.8. The downloaded version is saved in `cache/tools/llvm/bin`.

Sulong must be built with a more recent version of `clang` than 3.2, which means that `sulong` and `fastr` cannot be built in one step.
First make sure that you have a supported version of `clang` and related tools installed, e.g., 3.8 and that they are on your `PATH`. Also if you are on MacOS and are using MacPorts, you must make symbolic links to the explicitly versioned tool names, i.e. `clang` -> `clang-mp-3.8`. This also applies to the `opt` and `llvm-link` tools. You can set these links directly in `/opt/local/bin` using `sudo` or create a local `bin` directory and place the links there, making sure that this directory is on your `PATH`.

 and this (absolute) path should be added to `PATH`.

Now the remainder of sulong can be built.

    mx build

The `mx build` step will clone the `graal` repository, if necessary, and build the `truffle` suite also.

## Building FastR

To ensure that an LLVM build variant is chosen, set:

    export FASTR_RFFI=llvm

N.B. It is necessary to inform the build of the location of the various required libraries, e.g. `libpcre` as these are loaded explicitly from the `lib` directory at runtime. The build copies these libraries from wherever they were installed to the `lib` directory. For example on Ubuntu, set:

    export PKG_LDFLAGS_OVERRIDE=-L/lib/x86_64-linux-gnu

On Mac OS (with MacPorts) set:

    export PKG_LDFLAGS_OVERRIDE="-L/opt/local/lib -L/opt/local/lib/libgcc"

Now, ensure that `clang-3.2` is first on your `PATH` by adding the absolute path of `sulong/cache/tools/llvm/bin`.

Then run `mx build`.

## Running

There is no compile-time dependency between FastR and Sulong; all communication is via the Truffle Interop API. Therefore Sulong must be dynamically imported using either `mx --dynamicimport sulong` or by setting the environment variable `DEFAULT_DYNAMIC_IMPORTS=sulong`, with latter being most convenient. With this in effect, a normal `mx R` will make SuLong available.

Note that if the `LLVM_PARSE_TIME` environment variable is set to any value, the time taken to parse each LLVM module is logged to the console, which is also an indication that the LLVM implementation variant is being used.

# Implementation Details

## Compiler Wrapper Scripts

The compiler wrapper scripts are simple shell scripts that first test for the existence of the `sulong` sibling directory and, if it exists and the environment variable `FASTR_SULONG_IGNORE` is not set, invoke associated `mx` commands to perform the compilation. Otherwise, the standard compiler is used. The scripts are stored in the `compilers` sub-directory of `mx.fastr` and are named: `fastr-cc`, `fastr-fc`, `fastr-c++` and `fastr-cpp`. The associated `mx` commands are in `mx.fastr/mx_fastr_compilers.py`.

Currently, for convenience, the Python wrappers invoke code in the Sulong `sulong/mx.sulong` directory. Eventually, they will modified to be independent of Sulong.

## Limitations
At the time of writing all the `RFFI` interfaces are implemented for LLVM. However, owing to a bug in DragonEgg, the actual Fortran code for the Lapack library is not executed under LLVM, only the wrapper.
